import { LitElement } from 'lit';
declare const UUIRangeSliderElement_base: (new (...args: any[]) => import("@umbraco-ui/uui-base/lib/mixins").FormControlMixinInterface) & typeof LitElement;
/**
 * @element uui-range-slider
 * @description - Range slider with two handles. Use uui-slider for a single handle.
 */
export declare class UUIRangeSliderElement extends UUIRangeSliderElement_base {
    static styles: import("lit").CSSResult[];
    static readonly formAssociated = true;
    /**
     * Disables the input.
     * @type {boolean}
     * @attr
     * @default false
     */
    disabled: boolean;
    /**
     * Label to be used for aria-label and eventually as visual label. Adds " low value" and " high value" endings for the two values.
     * @type {string}
     * @attr
     */
    label: String;
    /**
     * This reflects the behavior of a native input step attribute.
     * @type {number}
     * @attr
     * @default 1
     */
    step: number;
    /**
     * Hides the numbers representing the value of each steps. Dots will still be visible
     * @type {boolean}
     * @attr 'hide-step-values'
     * @default false
     */
    hideStepValues: boolean;
    private _min;
    private _max;
    private _valueLow;
    private _valueHigh;
    private _minGap;
    private _maxGap?;
    /**
     * Sets the minimum allowed value.
     * @type {number}
     * @attr min
     * @default 0
     */
    set min(newVal: number);
    get min(): number;
    /**
     * Sets the maximum allowed value.
     * @type {number}
     * @attr max
     * @default 100
     */
    set max(newVal: number);
    get max(): number;
    /**
     * This is a value property of the uui-range-slider. Split the two values with comma, forexample 10,50 sets the values to 10 and 50.
     * @type {string}
     * @attr
     * @default 0,100
     */
    get value(): FormDataEntryValue | FormData;
    set value(newVal: FormDataEntryValue | FormData);
    /**
     * The lower picked value.
     * @type {number}
     * @attr value-low
     * @default 0
     */
    set valueLow(newVal: number);
    get valueLow(): number;
    /**
     * The higher picked value.
     * @type {number}
     * @attr value-high
     * @default 100
     */
    set valueHigh(newVal: number);
    get valueHigh(): number;
    /**
     * Minimum value gap between the the two picked values. Cannot be lower than the step value and cannot be higher than the maximum gap
     * @type {number}
     * @attr min-gap
     * @default 1
     */
    set minGap(newVal: number);
    get minGap(): number;
    /**
     * Maximum value gap between the the two picked values. Cannot be lower than the minimum gap.
     * @type {number}
     * @attr max-gap
     * @default undefined
     */
    set maxGap(newVal: number | undefined);
    get maxGap(): number | undefined;
    private _trackWidth;
    private _handle;
    private _inputLow;
    private _inputHigh;
    private _sliderTrack;
    private _innerSliderTrack;
    private _innerColor;
    focus(): void;
    protected getFormElement(): HTMLElement;
    private _onMinInput;
    private _onMaxInput;
    private _onChange;
    private _fillColor;
    private _onKeypress;
    private _onTouchStart;
    private _onTouchMove;
    private _onTouchEnd;
    private _onMouseDown;
    private _onMouseMove;
    private _onMouseUp;
    private stopMoving;
    private _getValue;
    private _getHandles;
    private _updateBothValues;
    constructor();
    updated(changedProperties: Map<string | number | symbol, unknown>): void;
    connectedCallback(): void;
    private _sliderLowThumbPosition;
    private _sliderHighThumbPosition;
    renderSteps(): import("lit-html").TemplateResult<2>;
    renderStepValues(hide: boolean): import("lit-html").TemplateResult<1>;
    private _onInputMouseDown;
    render(): import("lit-html").TemplateResult<1>;
}
declare global {
    interface HTMLElementTagNameMap {
        'uui-range-slider': UUIRangeSliderElement;
    }
}
export {};
